package MT::Plugin::ChangeCategoryCollectively;

use strict;
use base qw(MT::Plugin);
use MT;
use MT::Template::Context;
use vars qw($VERSION);
$VERSION = 1.00;

my $plugin = new MT::Plugin::ChangeCategoryCollectively({
	name        => "ChangeCategoryCollectively",
	id          => "ChangeCategoryCollectively",
	key         => "ChangeCategoryCollectively",
	description => "エントリー編集画面の 'その他の操作...' に、カテゴリーの操作を付け加えます。",
#	doc_link    => "http://hwat.sakura.ne.jp/hpod/spanners/movable_type_32/changecategorycollectively/",
#	plugin_link => "http://hwat.sakura.ne.jp/hpod/spanners/movable_type_32/changecategorycollectively/",
	author_name => "Yujiro Araki",               # hiroaki
	author_link => "http://www.koikikukan.com/", # http://homepage.mac.com/hwat/
	version     => $VERSION,
    l10n_class  => 'ChangeCategoryCollectively::L10N',
});
MT->add_plugin($plugin);

my %Category_label = ();


sub init_app {
	my $plugin = shift;
	my $app    = shift;
	return unless $app->isa('MT::App::CMS');
}

sub init_request {
	my $plugin = shift;
	my $app    = shift;
#	return unless $app->isa('MT::App::CMS');

    my $blog_id = $app->param('blog_id');
    return unless $blog_id;

    my $plugin = MT->component('ChangeCategoryCollectively');
    my $attach = $plugin->translate('attach');
    my $detach = $plugin->translate('detach');

    $app->add_itemset_action( {
        type        => 'entry',
        key         => 'ChangeCategoryCollectively',
        label       => $detach,
        code        => sub { $plugin->category_assignment($app, undef)   },
        condition   => sub { $plugin->perm_check($app)          },
        } );

    require MT::Category;
    for my $category ( MT::Category->load( {blog_id => $blog_id} ) ){
        $app->add_itemset_action( {
            type        => 'entry',
            key         => 'ChangeCategoryCollectively_'.$category->id,
            label       => $attach.$category->label,
            code        => sub { $plugin->category_assignment($app, $category->id)   },
            condition   => sub { $plugin->perm_check($app) },
            } );
        $Category_label{$category->id} = $category->label;
    }
}

sub instance { return $plugin; }

sub perm_check {
    my $plugin = shift;
    my ($app) = @_;
    my $perms = $app->{perms};
    $perms ? $perms->can_edit_all_posts : $app->user->is_superuser;
}

sub category_assignment {
    my $plugin = shift;
    my ($app, $cat_id) = @_;

    my $blog_id = $app->param('blog_id');
    return unless $blog_id;

    my @param_entries;

    require MT::Entry;
    require MT::Placement;
    for my $entry_id ( $app->param('id') ){

        my @placements          = ();   # buffer
        my @before_placements   = ();   # for informations
        my @after_placements    = ();   # for informations

        # load category list that entry has
        @placements = MT::Placement->load( {
            entry_id => $entry_id,
            } );

        # make info for 'before'
        for my $placement ( @placements ){
            push @before_placements, {
                placement_id => $placement->id,
                entry_id     => $placement->entry_id,
                category_id  => $placement->category_id,
                is_primary   => $placement->is_primary,
                category_label => $Category_label{$placement->category_id},
                };
        }

        unless( $cat_id ){
            # remove all placements
            for my $placement ( @placements ){
                $placement->remove;
            }
        }else{
            # check placement entry has or that is primary
            my $is_primary  = 1;
            my $is_found    = 0;
            for my $placement ( @placements ){
                if( $placement->is_primary ){
                    $is_primary = 0;
                }
                if( $placement->category_id == $cat_id ){
                    $is_found = 1;
                    last;
                }
            }
            
            unless( $is_found ){
                my $place = MT::Placement->new;
                $place->category_id( $cat_id     );
                $place->entry_id(    $entry_id   );
                $place->blog_id(     $blog_id    );
                $place->is_primary(  $is_primary );
                $place->save
                   or return $app->error($app->translate(
                        "Saving placement failed: [_1]", $place->errstr));
            }
        }

        # re-load category list
        @placements = MT::Placement->load( {
            entry_id => $entry_id,
            } );

        # make info 'after'
        for my $placement ( @placements ){
            push @after_placements, {
                placement_id => $placement->id,
                entry_id     => $placement->entry_id,
                category_id  => $placement->category_id,
                is_primary   => $placement->is_primary,
                category_label => $Category_label{$placement->category_id},
                };
        }


        # make parameters for view
        my $entry = MT::Entry->load( {
                        id => $entry_id,
                        } );
        push @param_entries, {
            tr_color    => (scalar @param_entries) % 2 ? 'even' : 'odd',
            entry_title => $entry->title,
            entry_id    => $entry_id,
            before_placements   => \@before_placements,
            after_placements    => \@after_placements,
            };

	}

    my $plugin = MT->component('ChangeCategoryCollectively');
    my $category = $plugin->translate('category');
    my $attached = $plugin->translate('attached');
    my $detached = $plugin->translate('detached');
    
    my $param = {};
#    $param->{result_title}  = $plugin->name . ' - ' . $plugin->version;
    $param->{result_header} = $cat_id
        ? $category . $Category_label{$cat_id} . $attached : $detached;
	$param->{param_entries}       = \@param_entries;
	$param->{return_url}    = $app->return_uri;
	$app->build_page( $plugin->load_tmpl('result.tmpl'), $param );
}


=head1 NAME

ChangeCategoryCollectively - エントリーのカテゴリーを一括して変更する

=head1 DESCRIPTION

この Movable Type (MT) 用のプラグインは、エントリーの編集の場面に、選択されたエントリーのカテゴリーを一括して変更するためのインタフェースを提供します。

エントリーのリストされた編集画面には「その他の操作... 」というプルダウンメニューがありますが、このプラグインをインストールすると、その内容に、カテゴリーを操作するメニューが新たに加わります。

=over4

=item カテゴリーをすべて外す

選択されたエントリーについて、カテゴリーをすべて削除します。

=item カテゴリーを付ける

選択されたエントリーについて、選ばれたカテゴリーを付加します。プライマリカテゴリーを持っていないエントリーは、選ばれたカテゴリーがプライマリになり、既にプライマリーカテゴリーを持っている場合は、サブカテゴリーとして追加します。

=back

あるカテゴリーだけを外すには、すべてを外してから、必要なカテゴリーをすべて追加してください。

ひとつのカテゴリーだけを外す機能は、構想にはありましたが、（このインタフェースでは）プルダウンが長くなり過ぎてしまうことをきらい、この機能の実装は見送られました。よりよいインタフェースのご提案がありましたら、ぜひコメントをおよせください。

=head1 SETUP

MT のプラグインフォルダに、この配布物に含まれているフォルダ plugins の中身（フォルダ ChangeCategoryCollectively ）を置いてください。

なお、 Movable Type 3.2-ja-2 での動作を確認していますが、ほかのバージョンでの動作は考慮されていません。

=head1 HISTORY

=over4

=item version 0.01 (2006-05-17)

最初のリリース。

選択されたエントリーの、カテゴリーをすべて外すメニューを追加。

選択されたエントリーの、（ブログに登録されているカテゴリーのなかから、ひとつの）カテゴリーを付加するメニューを追加。

=back

=head1 SEE ALSO

L<MT::App::CMS>

=head1 COPYRIGHT

Copyright 2006 WATANABE Hiroaki <hwat at mac dot com>

This library is free software; you can redistribute it
and/or modify it under the same terms as Perl itself.

=cut

1;
